@@ -4,7 +4,6 @@
 #include <iostream>
 #include <ft2build.h>
 #include FT_FREETYPE_H
-#include FT_TRUETYPE_TABLES_H
 #include "../FontDescriptor.h"
 #include "../utils.h"
 #include "../font_matching.h"
@@ -30,46 +29,16 @@ char *utf16ToUtf8(const WCHAR *input) {
   return output;
 }
 
-FontWeight get_font_weight(FT_Face face) {
-  void* table = FT_Get_Sfnt_Table(face, FT_SFNT_OS2);
-  if (table == NULL) {
-    return FontWeightUndefined;
-  }
-  TT_OS2* os2_table = (TT_OS2*) table;
-  return (FontWeight) os2_table->usWeightClass;
-}
-
-FontWidth get_font_width(FT_Face face) {
-  void* table = FT_Get_Sfnt_Table(face, FT_SFNT_OS2);
-  if (table == NULL) {
-    return FontWidthUndefined;
-  }
-  TT_OS2* os2_table = (TT_OS2*) table;
-  return (FontWidth) os2_table->usWidthClass;
-}
-FontDescriptor* descriptor_from_face(FT_Face &face, const char* path, int index) {
-  FontDescriptor* res = NULL;
-
-  res = new FontDescriptor(
-    path,
-    index,
-    FT_Get_Postscript_Name(face) == NULL ? "" : FT_Get_Postscript_Name(face),
-    face->family_name,
-    face->style_name,
-    get_font_weight(face),
-    get_font_width(face),
-    face->style_flags & FT_STYLE_FLAG_ITALIC,
-    FT_IS_FIXED_WIDTH(face)
-  );
-  return res;
-}
-
-int scan_font_dir(HKEY which, bool data_is_path) {
+int scan_font_dir(HKEY which, bool data_is_path, bool last_chance = false) {
   char win_dir[MAX_PATH];
   GetWindowsDirectoryA(win_dir, MAX_PATH);
 
   std::string font_dir;
-  font_dir += win_dir;
+  if (last_chance) {
+    font_dir = "C:\\WINDOWS";
+  } else {
+    font_dir += win_dir;
+  }
   font_dir += "\\Fonts\\";
 
   static const LPCSTR font_registry_path = "Software\\Microsoft\\Windows NT\\CurrentVersion\\Fonts";
@@ -126,7 +95,7 @@ int scan_font_dir(HKEY which, bool data_is_path) {
     if (error) {
       continue;
     }
-    font_list.push_back(descriptor_from_face(face, font_path.c_str(), 0));
+    font_list.push_back(new FontDescriptor(face, font_path.c_str(), 0));
     int n_fonts = face->num_faces;
     FT_Done_Face(face);
     for (int i = 1; i < n_fonts; ++i) {
@@ -137,7 +106,7 @@ int scan_font_dir(HKEY which, bool data_is_path) {
       if (error) {
         continue;
       }
-      font_list.push_back(descriptor_from_face(face, font_path.c_str(), i));
+      font_list.push_back(new FontDescriptor(face, font_path.c_str(), i));
       FT_Done_Face(face);
     }
   } while (result != ERROR_NO_MORE_ITEMS);
@@ -146,7 +115,7 @@ int scan_font_dir(HKEY which, bool data_is_path) {
   delete[] value_name;
   delete[] value_data;
   FT_Done_FreeType(library);
-  
+
   return 0;
 }
 int scan_font_reg() {
@@ -155,6 +124,11 @@ int scan_font_reg() {
 
   // Move Arial Regular to front
   ResultSet& font_list = get_font_list();
+
+  if (font_list.n_fonts() == 0) {
+    scan_font_dir(HKEY_LOCAL_MACHINE, false, true);
+  }
+
   for (ResultSet::iterator it = font_list.begin(); it != font_list.end(); it++) {
     if (strcmp((*it)->family, "Arial") == 0 && strcmp((*it)->style, "Regular") == 0) {
       FontDescriptor* arial = *it;
@@ -204,9 +178,6 @@ bool resultMatches(FontDescriptor *result, FontDescriptor *desc) {
   if (desc->italic != result->italic)
     return false;
 
-  if (desc->monospace != result->monospace)
-    return false;
-
   return true;
 }
 
@@ -227,6 +198,18 @@ ResultSet *findFonts(FontDescriptor *desc) {
 FontDescriptor *findFont(FontDescriptor *desc) {
   ResultSet *fonts = findFonts(desc);
 
+  // if we didn't find anything, try again with postscriptName as family
+  if (fonts->size() == 0) {
+    delete fonts;
+
+    desc->postscriptName = desc->family;
+    desc->family = NULL;
+
+    fonts = findFonts(desc);
+
+    desc->family = desc->postscriptName;
+    desc->postscriptName = NULL;
+  }
   // if we didn't find anything, try again with only the font traits, no string names
   if (fonts->size() == 0) {
     delete fonts;
@@ -269,7 +252,7 @@ bool font_has_glyphs(const char * font_path, int index, FT_Library &library, uin
   if (error) {
     return false;
   }
-  
+
   bool has_glyph = false;
   for (int i = 0; i < n_chars; ++i) {
     if (FT_Get_Char_Index( face, str[i])) {
@@ -277,7 +260,7 @@ bool font_has_glyphs(const char * font_path, int index, FT_Library &library, uin
       break;
     }
   }
-  
+
   FT_Done_Face(face);
   return has_glyph;
 }
@@ -287,36 +270,36 @@ int scan_link_reg() {
   if (font_links.size() != 0) {
     return 0;
   }
-  
+
   static const LPCSTR link_registry_path = "Software\\Microsoft\\Windows NT\\CurrentVersion\\FontLink\\SystemLink";
   HKEY h_key;
   LONG result;
-  
+
   result = RegOpenKeyExA(HKEY_LOCAL_MACHINE, link_registry_path, 0, KEY_READ, &h_key);
   if (result != ERROR_SUCCESS) {
     return 1;
   }
-  
+
   DWORD max_value_name_size, max_value_data_size;
   result = RegQueryInfoKey(h_key, 0, 0, 0, 0, 0, 0, 0, &max_value_name_size, &max_value_data_size, 0, 0);
   if (result != ERROR_SUCCESS) {
     return 1;
   }
-  
+
   DWORD value_index = 0;
   LPSTR value_name = new CHAR[max_value_name_size];
   LPBYTE value_data = new BYTE[max_value_data_size];
   DWORD value_name_size, value_data_size, value_type;
-  
+
   do {
     // Loop over font registry, construct file path and parse with freetype
     value_data_size = max_value_data_size;
     value_name_size = max_value_name_size;
-    
+
     result = RegEnumValueA(h_key, value_index, value_name, &value_name_size, 0, &value_type, value_data, &value_data_size);
-    
+
     value_index++;
-    
+
     if (!(result == ERROR_SUCCESS || result == ERROR_MORE_DATA) || value_type != REG_MULTI_SZ) {
       continue;
     }
@@ -347,22 +330,22 @@ int scan_link_reg() {
       }
       ++value_counter;
     }
-    
+
     if (values.size() != 0) {
       font_links[name] = values;
     }
   } while (result != ERROR_NO_MORE_ITEMS);
-  
+
   // Cleanup
   delete[] value_name;
   delete[] value_data;
-  
+
   return 0;
 }
 
 FontDescriptor *substituteFont(char *postscriptName, char *string) {
   scan_link_reg();
-  
+
   FontDescriptor *res = NULL;
   // find the font for the given postscript name
   FontDescriptor *desc = new FontDescriptor();
@@ -373,7 +356,7 @@ FontDescriptor *substituteFont(char *postscriptName, char *string) {
     delete desc;
     return font;
   }
-  
+
   FT_Library library;
   FT_Error    error;
   error = FT_Init_FreeType( &library );
@@ -381,20 +364,20 @@ FontDescriptor *substituteFont(char *postscriptName, char *string) {
     delete desc;
     return font;
   }
-  
+
   UTF_UCS conv;
   int n_chars = 0;
-  
+
   uint32_t* str = conv.convert(string, n_chars);
-  
+
   // Does the provided one work?
   if (font->path != NULL && font_has_glyphs(font->path, font->index, library, str, n_chars)) {
     FT_Done_FreeType(library);
     delete desc;
-    
+
     return font;
   }
-  
+
   // Try emoji
   desc->family = EMOJI;
   res = findFont(desc);
@@ -403,19 +386,19 @@ FontDescriptor *substituteFont(char *postscriptName, char *string) {
     FT_Done_FreeType(library);
     delete desc;
     delete font;
-    
+
     return res;
   }
   delete res;
-  
+
   desc->weight = font->weight;
   desc->italic = font->italic;
-  
+
   // Look for links
   WinLinkMap& font_links = get_win_link_map();
   std::string family(font->get_family());
   auto link = font_links.find(family);
-  
+
   // If the font doesn't have links, try the different standard system fonts
   if (link == font_links.end()) {
     link = font_links.find("Segoe UI");
@@ -426,7 +409,7 @@ FontDescriptor *substituteFont(char *postscriptName, char *string) {
       }
     }
   }
-  
+
   // hopefully some links were found
   if (link != font_links.end()) {
     for (auto it = link->second.begin(); it != link->second.end(); ++it) {
@@ -437,13 +420,13 @@ FontDescriptor *substituteFont(char *postscriptName, char *string) {
         FT_Done_FreeType(library);
         delete desc;
         delete font;
-        
+
         return res;
       }
       delete res;
     }
   }
-  
+
   // Still no match -> try some standard unicode fonts
   static std::vector<std::string> fallbacks = {
     "Segoe UI", // Latin, Greek, Cyrillic, Arabic
@@ -453,11 +436,24 @@ FontDescriptor *substituteFont(char *postscriptName, char *string) {
     "MS UI Gothic", // CJK (Japanese)
     "Microsoft JhengHei UI", // CJK (Traditional Chinese)
     "Microsoft YaHei UI", // CJK (Simplified Chinese)
+    "Microsoft Himalaya", // Tibetan
+    "Microsoft New Tai Lue", // New Tai Lue
+    "Microsoft PhagsPa", // Phags-Pa
+    "Microsoft Sans Serif", // Latin, Greek, Cyrillic, Arabic, Hebrew, Thai, Vietnamese, Baltic, Turkish
+    "Microsoft Tai Le", // Tai Le
+    "Microsoft Yi Baiti", // Yi
+    "Nirmala UI", // Many central asian scripts
     "Malgun Gothic", // CJK (Korean)
     "PMingLiU", // CJK (Traditional Chinese)
     "SimSun", // CJK (Simplified Chinese)
     "Gulim", // CJK (Korean)
     "Yu Gothic", // CJK (Japanese)
+    "Leelawadee UI", // Thai
+    "Ebrima", // African
+    "Gadugi", // Cherokee
+    "Jawanese Text", // Javanese
+    "Mongolian Baiti", // Mongolian
+    "Myanmar Text", // Myanmar
     "Segoe UI Symbol"// Symbols
   };
   for (auto it = fallbacks.begin(); it != fallbacks.end(); ++it) {
@@ -468,14 +464,14 @@ FontDescriptor *substituteFont(char *postscriptName, char *string) {
       FT_Done_FreeType(library);
       delete desc;
       delete font;
-      
+
       return res;
     }
     delete res;
   }
-  
+
   // Really? We just return the input font
-  
+
   FT_Done_FreeType(library);
   delete desc;
   return font;
